"""
The `audiodotturn` package contains the source code for AudioDotTurn,
a tool for formatting and organizing audio files with no metadata.

Modules:
- `cli_runner`: defines the command line interface for AudioDotTurn
- `commands`: defines the `create` and `view` subcommands for AudioDotTurn
- `config`: contains the `Config` class, which reads and writes configuration data
- `parsers`: defines parsers for parsing metadata from audio files

Constants:
- `VERSION`: the version of the AudioDotTurn package
"""

__version__ = VERSION = "v0.3.4"
